package controllers;

import java.io.IOException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.UserDAO;
import sun.misc.BASE64Decoder;
import sun.misc.BASE64Encoder;
import utils.Security;
import utils.ValidationUtils;
import dto.Configuration;
import dto.User;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/configuration")
public class ConfigurationController {

	private @Autowired UserDAO userDAO;
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/get", method=RequestMethod.GET)
	@ResponseBody
	public User getUser(String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					String userSid = Security.getUsername(hash);
					User user = userDAO.get(userSid);
					// Puts null params that we don't need
					user.setEndedDate(null);
					user.setRecordDate(null);
					byte[] password = new BASE64Decoder().decodeBuffer(user.getPassword());
					String decodePassword = new String(password);
					user.setPassword(decodePassword);
					return user;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/edit", method=RequestMethod.POST)
	@ResponseBody
	public Configuration editData(String email, String password, String hash) throws UnauthorizedException, ValidationException, IOException{
		if (!ValidationUtils.isEmptyOrNull(email)
				&& !ValidationUtils.isEmptyOrNull(password)
				&& !ValidationUtils.isEmptyOrNull(hash)){
			try{
				if (Security.existHash(hash, userDAO)){
					
					Configuration configuration = new Configuration();
					String userSid = Security.getUsername(hash);
					
					User user = userDAO.get(userSid);
					if (!user.getEmail().equals(email)){
						if (!userDAO.emailAvailable(email)){
							configuration.setNotAvailableEmail(true);
						}
					}
					if (!configuration.isNotAvailableEmail()){
						userDAO.editEmail(userSid, email);
						String encodedPassword = new BASE64Encoder().encode(password.getBytes());
						userDAO.editPassword(userSid, encodedPassword);
						
						// Hash
						String noEncodedHash = userSid+":"+encodedPassword;
						hash = new BASE64Encoder().encode(noEncodedHash.getBytes());
						configuration.setHash(hash);
					}
					
					return configuration;
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
		
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
