package controllers;

import java.io.IOException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.UserDAO;
import sun.misc.BASE64Encoder;
import utils.Security;
import utils.ValidationUtils;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/login")
public class LoginController {

	private @Autowired UserDAO userDAO;
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/checkLocalStorage", method=RequestMethod.POST)
	@ResponseBody
	public void checkLoginLocalStorage(String hash) throws ValidationException, UnauthorizedException, IOException{
		if (!ValidationUtils.isEmptyOrNull(hash)){
			try{
				boolean isLogged = Security.existHash(hash, userDAO);
				if (!isLogged){
					throw new UnauthorizedException("Hash not valid");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
			catch (IOException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@Transactional(readOnly = true)
	@RequestMapping(value="/check", method=RequestMethod.POST)
	@ResponseBody
	public String checkLogin(String username,
			String password) throws UnauthorizedException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(username)
				&& !ValidationUtils.isEmptyOrNull(password)){
			try{
				String encodedPassword = new BASE64Encoder().encode(password.getBytes());
				boolean isLogged = userDAO.exists(username, encodedPassword);
				if (isLogged){
					// Return hash
					String noEncodedHash = username+":"+encodedPassword;
					String hash = new BASE64Encoder().encode(noEncodedHash.getBytes());
					return hash;
				}
				else{
					return null;
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	@RequestMapping(value="/show", method=RequestMethod.GET)
	@ResponseBody
	public String showUsername(String hash) throws IOException{
		return Security.getUsername(hash);
	}
	
	// Errors
	
	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
	
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
	
}
