package controllers;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.UserDAO;
import sun.misc.BASE64Encoder;
import utils.ValidationUtils;
import dto.Pair;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/register")
public class RegisterController {
	
	@Autowired UserDAO userDAO;
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/new", method=RequestMethod.POST)
	@ResponseBody
	public Pair<Boolean, String> registerUser(String username, String email, String password) throws ValidationException{
		if (!ValidationUtils.isEmptyOrNull(username)
				&& !ValidationUtils.isEmptyOrNull(email)
				&& !ValidationUtils.isEmptyOrNull(password)){
			try{
				Pair<Boolean, String> pair = new Pair<Boolean,String>();
				if (userDAO.usernameAndEmailAvailable(username, email)){
					String encodedPassword = new BASE64Encoder().encode(password.getBytes());
					userDAO.create(username, email, encodedPassword);
					
					// Hash
					String noEncodedHash = username+":"+encodedPassword;
					String hash = new BASE64Encoder().encode(noEncodedHash.getBytes());
					
					pair.setA(true);
					pair.setB(hash);
				}
				else{
					pair.setA(false);
				}
				return pair;
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}
	}
	
	// Errors
	
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
