package controllers;

import java.io.IOException;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Controller;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import services.FilmCandidateDAO;
import services.MeetingDAO;
import services.MemberDAO;
import services.NotificationDAO;
import services.UserDAO;
import utils.Security;
import utils.ValidationUtils;
import dto.Member;
import exceptions.UnauthorizedException;
import exceptions.ValidationException;


@Controller
@RequestMapping(value="/vote")
public class VoteController {

	private @Autowired UserDAO userDAO;
	private @Autowired FilmCandidateDAO filmCandidateDAO;
	private @Autowired MemberDAO memberDAO;
	private @Autowired MeetingDAO meetingDAO;
	private @Autowired NotificationDAO notificationDAO;
	
	@Transactional(readOnly = false)
	@RequestMapping(value="/accept", method=RequestMethod.POST)
	@ResponseBody
	public void vote(int meetingSid, @RequestParam("filmSidList[]") List<Integer> filmSidList, String hash) throws IOException, UnauthorizedException, ValidationException{
		if (!ValidationUtils.isEmptyOrNull(filmSidList)
				&& !ValidationUtils.isEmptyOrNull(hash)){	
			try{
				if (Security.existHash(hash, userDAO)){
					String userSid = Security.getUsername(hash);
					// Votes films
					for (int filmSid : filmSidList){
						filmCandidateDAO.voteFilm(meetingSid, filmSid);
					}
 					
					// Updates my member status
					memberDAO.updateVoted(meetingSid, userSid);
					
					// Checks if all members have voted
					boolean voted = memberDAO.checkAllVoted(meetingSid);
					if (voted){
						meetingDAO.updateVoted(meetingSid);
						
						// Gets all members
						List<Member> memberList = memberDAO.getAll(meetingSid);
						for (Member member : memberList){
							notificationDAO.createMeeting(userSid, member.getUserSid(), meetingSid, true);
						}
					}
				}
				else{
					throw new UnauthorizedException("Not logged");
				}
			}
			catch (DataAccessException e) {
				throw e;
			}
		}
		else{
			throw new ValidationException("Parameters not valid");
		}

	}
	
	// Errors

	@ExceptionHandler(IOException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleIoException(){}
		
	@ExceptionHandler(DataAccessException.class)
	@ResponseStatus(value=HttpStatus.SERVICE_UNAVAILABLE)
	public void handleDataAccessException(){}
	
	@ExceptionHandler(UnauthorizedException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleUnauthorizedException(){}
	
	@ExceptionHandler(ValidationException.class)
	@ResponseStatus(value=HttpStatus.UNAUTHORIZED)
	public void handleValidationException(){}
}
