package services;

import java.sql.Timestamp;
import java.util.List;

import org.springframework.dao.DataAccessException;

import dto.Meeting;

public interface MeetingDAO {
	
	/**
	 * Creates a new meeting
	 * @param name String
	 * @param userSid String
	 * @param numberCandidates int
	 * @param inCinema boolean
	 * @param notSeen boolean
	 * @param date Timestamp
	 * @return int sid returned
	 * @throws DataAccessException
	 */
	int create(String name, String userSid, int numberCandidates, boolean inCinema, boolean notSeen, Timestamp date) throws DataAccessException;
	
	/**
	 * Gets a meeting
	 * @param sid int
	 * @return Meeting returned
	 * @throws DataAccessException
	 */
	Meeting get(int sid) throws DataAccessException;
	
	/**
	 * Gets the next meetings voted of a user
	 * @param userSid String
	 * @return List<Meeting> returned
	 * @throws DataAccessException
	 */
	List<Meeting> getNextMeetings(String userSid) throws DataAccessException;
	
	/**
	 * Gets the next meetings voted or not of a user
	 * @param userSid String
	 * @return List<Meeting> returned
	 * @throws DataAccessException
	 */
	List<Meeting> myMeetings(String userSid) throws DataAccessException;
	
	/**
	 * Updates voted
	 * @param sid int
	 * @throws DataAccessException
	 */
	void updateVoted(int sid) throws DataAccessException;
}
