package services.impl;


import java.sql.Timestamp;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import services.FilmLibraryDAO;
import dto.FilmLibrary;
 
@Repository
public class FilmLibraryDAOImpl implements FilmLibraryDAO{
	
	private @Autowired GeneralDAO generalDAO;

	@Transactional(readOnly = false)
	public void add(int filmSid, String userSid, boolean gotIt, Boolean seeWannaSee) throws DataAccessException {
		try{
			String query = "INSERT INTO filmlibrary (film_sid, user_sid, got_it, see_wannasee, record_date) VALUES (?, ?, ?, ?, ?)";
			Timestamp recordDate = new Timestamp(System.currentTimeMillis());
			Object[] params = new Object[]{
				filmSid, userSid, gotIt, seeWannaSee, recordDate
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = false)
	public void update(int filmSid, String userSid, boolean gotIt, Boolean seeWannaSee) throws DataAccessException {
		try{
			String query = "UPDATE filmlibrary SET got_it=?, see_wannasee=? WHERE user_sid=? AND film_sid=?";
			Object[] params = new Object[]{
				gotIt, seeWannaSee, userSid, filmSid
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public FilmLibrary get(int filmSid, String userSid) throws DataAccessException {
		try{
			String query = "SELECT * FROM filmlibrary WHERE film_sid=? And user_sid=?";
			Object[] params = new Object[]{
				filmSid, userSid
			};
			FilmLibrary filmLibrary = generalDAO.getJdbcTemplate().queryForObject(query, params, new BeanPropertyRowMapper<FilmLibrary>(FilmLibrary.class));
			return filmLibrary;
		}
		catch (EmptyResultDataAccessException e) {
			return null;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
}