package services.impl;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import services.FriendDAO;
 
@Repository
public class FriendDAOImpl implements FriendDAO{
	
	private @Autowired GeneralDAO generalDAO;
	
	@Transactional(readOnly = false)
	public void create(String userSid1, String userSid2) throws DataAccessException{
		try{
			String query = "INSERT INTO friend (user_sid1, user_sid2) VALUES (?, ?)";
			Object[] params = new Object[]{
				userSid1, userSid2
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = false)
	public void block(String userLogged, String userSid2, boolean block) throws DataAccessException{
		try{
			String query = "UPDATE friend SET blocker=? WHERE (user_sid1=? AND user_sid2=?) OR (user_sid1=? AND user_sid2=?)";
			String blocker = null;
			if (block){
				blocker = userLogged;
			}
			Object[] params = new Object[]{
				blocker, userLogged, userSid2, userSid2, userLogged
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public String getBlocker(String userSid1, String userSid2) throws DataAccessException{
		try{
			String query = "SELECT blocker FROM friend WHERE (user_sid1=? AND user_sid2=?) OR (user_sid1=? AND user_sid2=?)";
			Object[] params = new Object[]{
				userSid1, userSid2, userSid2, userSid1
			};
			return generalDAO.getJdbcTemplate().queryForObject(query, params, String.class);
		}
		catch (EmptyResultDataAccessException e) {
			return null;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public boolean exists(String userSid1, String userSid2) throws DataAccessException{
		try{
			String query = "SELECT COUNT(*) FROM friend WHERE (user_sid1=? AND user_sid2=?) OR (user_sid1=? AND user_sid2=?)";
			Object[] params = new Object[]{
				userSid1, userSid2, userSid2, userSid1
			};
			return generalDAO.getJdbcTemplate().queryForObject(query, params, Integer.class) == 1;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
}