package services.impl;


import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import services.MemberDAO;
import dto.Member;
 
@Repository
public class MemberDAOImpl implements MemberDAO{
	
	private @Autowired GeneralDAO generalDAO;
	
	@Transactional(readOnly = false)
	public void create(int meetingSid, String userSid, boolean gotIt) throws DataAccessException{
		try{
			String query = "INSERT INTO member (meeting_sid, user_sid, got_it, voted) VALUES (?, ?, ?, ?)";
			Object[] params = new Object[]{
				meetingSid, userSid, gotIt, false	
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public Member get(int meetingSid, String userSid) throws DataAccessException{
		try{
			String query = "SELECT * FROM member WHERE meeting_sid = ? AND user_sid = ?";
			Object[] params = new Object[]{
				meetingSid, userSid	
			};
			Member member = generalDAO.getJdbcTemplate().queryForObject(query, params, new BeanPropertyRowMapper<Member>(Member.class));
			return member;
		}
		catch (EmptyResultDataAccessException e) {
			return null;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
	
	@Transactional(readOnly = true)
	public List<Member> getAll(int meetingSid) throws DataAccessException{
		try{
			String query = "SELECT * FROM member WHERE meeting_sid = ?";
			Object[] params = new Object[]{
				meetingSid
			};
			List<Member> memberList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Member>(Member.class));
			return memberList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = false)
	public void updateVoted(int meetingSid, String userSid) throws DataAccessException {
		try{
			String query = "UPDATE member SET voted=true " +
					"WHERE member.meeting_sid=? AND member.user_sid=?";
			Object[] params = new Object[]{
				meetingSid, userSid
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = true)
	public boolean checkAllVoted(int meetingSid) throws DataAccessException {
		try{
			String query = "SELECT COUNT(*) FROM member WHERE meeting_sid=? AND voted=false";
			Object[] params = new Object[]{
				meetingSid
			};
			return generalDAO.getJdbcTemplate().queryForObject(query, params, Integer.class) == 0;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
}