package services.impl;


import java.sql.Timestamp;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import services.NotificationDAO;
import dto.Notification;
 
@Repository
public class NotificationDAOImpl implements NotificationDAO{
	
	private @Autowired GeneralDAO generalDAO;

	@Transactional(readOnly = false)
	public void createMeeting(String senderSid, String receiverSid, int meetingSid, boolean voted)
			throws DataAccessException {
		try{
			String query = "INSERT INTO notification (sender_sid, receiver_sid, meeting_sid, meeting_voted, seen, record_date) VALUES (?, ?, ?, ?, ?, ?)";
			Timestamp recordDate = new Timestamp(System.currentTimeMillis());
			Object[] params = new Object[]{
				senderSid, receiverSid, meetingSid, voted, false, recordDate
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = false)
	public void createNewFriend(String senderSid, String receiverSid)
			throws DataAccessException {
		try{
			String query = "INSERT INTO notification (sender_sid, receiver_sid, seen, record_date) VALUES (?, ?, ?, ?)";
			Timestamp recordDate = new Timestamp(System.currentTimeMillis());
			Object[] params = new Object[]{
				senderSid, receiverSid, false, recordDate
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = true)
	public List<Notification> getNotifications(String userSid) throws DataAccessException {
		try{
			String query = "SELECT * FROM notification WHERE receiver_sid=? ORDER BY record_date DESC";
			Object[] params = new Object[]{
				userSid	
			};
			List<Notification> notificationList = generalDAO.getJdbcTemplate().query(query, params, new BeanPropertyRowMapper<Notification>(Notification.class));
			return notificationList;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = false)
	public void seen(int notificationSid) throws DataAccessException {
		try{
			String query = "UPDATE notification SET seen=true, record_date=record_date WHERE sid=?";
			Object[] params = new Object[]{
				notificationSid
			};
			generalDAO.getJdbcTemplate().update(query, params);
		}
		catch (DataAccessException e) {
			throw e;
		}
	}

	@Transactional(readOnly = true)
	public boolean news(String userSid)
			throws DataAccessException {
		try{
			String query = "SELECT COUNT(*) FROM notification WHERE receiver_sid=? AND seen=false";
			Object[] params = new Object[]{
				userSid
			};
			return generalDAO.getJdbcTemplate().queryForObject(query, params, Integer.class) > 0;
		}
		catch (DataAccessException e) {
			throw e;
		}
	}
}